@@ -80,6 +80,12 @@ svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
            ids<-na.id(data.frame(ids))
        }
 
+    ## make ids factor if they are character
+    for(i in 1:ncol(ids)){
+        if (is.character(ids[[i]]))
+            ids[[i]]<-factor(ids[[i]])
+    }
+    
     na.prob<-na.failsafe("missing values in `prob'")
     if(inherits(probs,"formula")){
       mf<-substitute(model.frame(probs,data=data,na.action=na.prob))
@@ -110,6 +116,10 @@ svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
       if (!is.list(strata))
         strata<-data.frame(strata=strata)
       has.strata<-TRUE
+      for(i in 1:NCOL(strata)){ ##drop empty strata
+          if (is.factor(strata[[i]]))
+              strata[[i]]<-as.factor(as.character(strata[[i]]))
+          }
     } else {
       has.strata <-FALSE
       strata<-na.strata(as.data.frame(matrix(1, nrow=NROW(ids), ncol=NCOL(ids))))
@@ -267,6 +277,8 @@ onestrat<-function(x,cluster,nPSU,fpc, lonely.psu,stratum=NULL,stage=1,cal=cal){
 
 
 onestage<-function(x, strata, clusters, nPSU, fpc, lonely.psu=getOption("survey.lonely.psu"),stage=0, cal){
+   if (NROW(x)==0)
+        return(matrix(0,NCOL(x),NCOL(x)))
   stratvars<- tapply(1:NROW(x), list(factor(strata)), function(index){
              onestrat(x[index,,drop=FALSE], clusters[index],
              nPSU[index][1], fpc[index], ##changed from fpc[index][1], to allow pps(brewer)
